c
      program maxwell
c
c******************************************************************
c  REFERENCE:
c  Smith, B. and D. Sandwell, A three-dimensional semianalytic viscoelastic 
c  model for time-dependent analysis of the earthquake cycle,
c  J. Geophys. Res., v. 109, B12401, 2004.
c******************************************************************
c
c modified 04/13/07
c add gravity to common block
c******************************************************************

c
c  USAGE:
c  Program to calculate the surface displacement or stress
c  due to planar vertical fault consisting of many elements
c  where the slip varies on each element.  The model consists 
c  of an elastic plate of thickness H over a visco-elastic half space.
c  All elements slip between depths D1 and D2 where D1 must be
c  above the base of the plate. One can simulate either a single-couple
c  or double couple dislocation. The displacement or stress 
c  can be observed at any depth above the lower locking depth.
c
c  EXAMPLE:
c  An example of a double-couple fault extending from the base of the
c  plate to a depth of 20 km follows:
c
c******************************************************************
c  maxwell -50 -50 -20 0 99 1 all_segs.dat 0 xall.grd yall.grd zall.grd
c******************************************************************
c
c  The three components of surface displacement are stored
c  in three grd-files for use with GMT software.
c
c  Fault elements are stored in the file all_segs.dat
c
c******************************************************************
c  # dx  sig fault_orientation
c     1. 1.  90.
c  549.36 549.36   0.00 128.53 -40.00   0.00   0.00
c  549.36 549.62 128.53 132.51 -40.00   0.00   0.00
c  549.62 549.88 132.51 136.48 -40.00   0.00   0.00
c  549.88 550.26 136.48 142.45 -40.00   0.00   0.00
c  550.26 551.43 142.45 149.49 -40.00   0.00   0.00
c  551.43 552.47 149.49 154.55 -40.00   0.00   0.00
c******************************************************************
c
c  The first line is a comment line.  
c  The second line has; grid spacing(km), fault thickness(km), and average
c  fault orientation w.r.t the x-axis
c  Subsequent lines have the end points of the fault followed by slip
c  rates; strike-slip, dip-slip, and opening. Millions of elements
c  could be used without a change in memory or execution time.
c
c  A mirror fault is constructed to match the
c  the far-field boundary conditions. For an infinitely-long, straight fault,
c  this program generates the arctangent model. 
c
c*****************   main program   ************************
c
      implicit real*8 (a,b,d-h,o-z)
      implicit complex*16 (c)
      real*8 kx,ky
c
c**User: change ni and nj as needed*************************
      parameter(ni=2048,nj=2048,nwork=32768,nj2=nj/2+1,ni2=ni/2+1)
c*************************************************************
      parameter (nl=140)
      character*80 felement,fxdisp,fydisp,fzdisp
      character*80 cfac,cd1,cd2,cz,ct,cdbl,cstr
      character*80 cthk,ceta,cpois,cfd
      real*8 An(nl),rla(nl),rmu(nl)
c
      common/plate/rlam1,rlam2,rmu1,rmu2,rho,rc,rd,alph,pi,grv
c
c  the working arrays are all real*4, complex*8 to save memory
c
      real*4 fx(nj,ni),fy(nj,ni),fz(nj,ni),fm(nj,ni)
      real*4 u(nj,ni),v(nj,ni),w(nj,ni)
      complex*8 fkx(nj2,ni),fky(nj2,ni),fkz(nj2,ni)
      complex*8 uk(nj2,ni),vk(nj2,ni),wk(nj2,ni)
      dimension n(2)
      complex*8 work(nwork)
      equivalence (fx(1,1),fkx(1,1))
      equivalence (fy(1,1),fky(1,1))
      equivalence (fz(1,1),fkz(1,1))
      equivalence (u(1,1),uk(1,1))
      equivalence (v(1,1),vk(1,1))
      equivalence (w(1,1),wk(1,1))
c
c*********User: set these default model parameters************

      rho=3300.
      fr=0.6
      young=7.5e10
      pois=0.25
      eta=1.e19
      thk=-50.
      fd=1.0
      grv=-9.8
c*************************************************************
c
      pi=acos(-1.)
      spyr=86400*365.25
c
c  zero the arrays fx,fy,fz
c
      do 30 i=1,ni
      do 30 j=1,nj
      fx(j,i)=0.
      fy(j,i)=0.
      fz(j,i)=0.
  30  continue
c
c  set the dimensions for fourt
c
      n(1)=nj
      n(2)=ni
c
c   get values from command line
c
      narg = iargc()
      if(narg.lt.11) then
        write(*,'(a)')'  '
        write(*,'(a)')
     &  'Usage: maxwell F D1 D2 Z T dble ele.dat istr U.grd V.grd W.grd'
        write(*,'(a)')
        write(*,'(a)')
     &  '       F       - factor to multiply output files'
        write(*,'(a)')
     &  '       D1, D2  - depth to bottom and top of fault (neg) '
        write(*,'(a)')
     &  '       Z       - depth of observation plane '
        write(*,'(a)')
     &  '       T       - time since earthquake (yr) '
        write(*,'(a)')
     &  '       dble    - (0-single couple, 1-double couple)'
        write(*,'(a)')
     &  '       ele.dat - file of: x1,x2,y1,y2,F1,F2,F3'
        write(*,'(a)')
     &  '       istress - (0)-disp. U,V,W'
        write(*,'(a)')
     &  '                 (1)-stress Txx,Tyy,Txy'
        write(*,'(a)')
     &  '                 (2)-stress Tnormal,Tshear,Tcoulomb'
        write(*,'(a)')
     &  '       x,y,z.grd - output files of disp. or stress '
        write(*,'(a)')
     &  '       H       - plate thickness (default: H=-50km) - optional'
        write(*,'(a)')
     &  '       eta     - viscosity (default: eta=1.e19) - optional'
        write(*,'(a)')
     &  '       pois	- poissons ratio (default: pois=0.25) - optional'
        write(*,'(a)')
     &  '       fd	- depth factor (default: fd=1) - optional'
        write(*,'(a)')'  '
        stop
      else 
        call getarg(1,cfac)
        call getarg(2,cd1)
        call getarg(3,cd2)
        call getarg(4,cz)
        call getarg(5,ct)
        call getarg(6,cdbl)
        call getarg(7,felement)
        call getarg(8,cstr)
        read(cfac,*)fac
        read(cd1,*)d1
        read(cd2,*)d2
        read(cz,*)zobs
        read(ct,*)timey
        time=spyr*timey
	read(cdbl,*)idble
        read(cstr,*)istr
	
        call getarg(9,fxdisp)
        nc=index(fxdisp,' ')
        fxdisp(nc:nc)=char(0)
        call getarg(10,fydisp)
        nc=index(fydisp,' ')
        fydisp(nc:nc)=char(0)
        call getarg(11,fzdisp)
        nc=index(fzdisp,' ')
        fzdisp(nc:nc)=char(0)
        if(narg.eq.15) then
          call getarg(12,cthk)
          call getarg(13,ceta)
          call getarg(14,cpois)
          call getarg(15,cfd)
          read(cthk,*)thk
          read(ceta,*)eta
          read(cpois,*)pois
          read(cfd,*)fd

        endif	  
c
c   scale d1 by fd for double couple models only
c
          if(idble.eq.1) d1=d1*fd
          write(*,*)'H,eta,pois,d1,d2 = ',thk,eta,pois,d1,d2
        endif
	  if(thk.gt.d1.or.d1.gt.d2.or.d2.gt.0.or.
     +     zobs.lt.d1) then
         write(*,'(a)')' depths  must be  negative H < d1 < d2 '
         stop
      endif
c
c  open the input file and load the force arrays
c
      open(unit=5,file=felement,status='old')
      read(5,*)cd1
c
c  read the fault constants
c
      read(5,*) dx,sig,psi
c
      width=dx*nj
      height=dx*ni
      if(pois.ge.0.5) then
        write(*,'(a)')' error in Poisson ratio '
        stop
      endif
      rlam1=young*pois/((1.+pois)*(1.-2.*pois))
      rmu1=0.5*young/(1.+pois)
      alph=(rlam1+rmu1)/(rlam1+2*rmu1)
      rc=alph/(4.*rmu1)
      rd=(rlam1+3*rmu1)/(rlam1+rmu1)
      sig=sig/dx
      if(sig.lt.1)sig=1
c
c  read the elements
c
      nF2=0
      nF3=0
   50 read(5,*,end=999)x1,x2,y1,y2,F1,F2,F3
c
c is either F2 or F3 being used?
c
      if(F2.ne.0.0)nF2=nF2+1
      if(F3.ne.0.0)nF3=nF3+1
      x1=x1/dx
      x2=x2/dx
c
c  invert the y-positions to allign with i-index
c
      y1=ni-y1/dx
      y2=ni-y2/dx
c
c  figure out the part of the array to add the new forces
c
      ipd=3*sig+2
      jx0=min(x1,x2)-ipd
      jxf=max(x1,x2)+.5+ipd
      iy0=min(y1,y2)-ipd
      iyf=max(y1,y2)+.5+ipd
      do 100 iy=iy0,iyf
      do 200 jx=jx0,jxf
        if(jx.le.0.or.jx.gt.nj) go to 200
        if(iy.le.0.or.iy.gt.ni) go to 200
        x=jx
        y=iy
        call element(x,y,x1,y1,x2,y2,dx,sig,idble,F1,F2,F3,fxx,fyy,fzz)
c
c remove the x-forces if the fault is single couple
c
        fx(jx,iy)=fx(jx,iy)+fxx
        fy(jx,iy)=fy(jx,iy)+fyy
        fz(jx,iy)=fz(jx,iy)+fzz
        if(idble.eq.0) fx(jx,iy)=0.0
 200  continue
 100  continue
      go to 50
 999  close(unit=5)
c
c  send a warning about F2 and F3
c
      if(nF2.gt.0.or.nF3.gt.0) write(*,'(a)')
     &  ' warning: dip-slip (F2) and opening (F3) assume single couple'

c
c  mirror the source in the x-direction
c  this will make the finite displacement BC's match
c  one could also use a cosine transform
c
      do 150 iy=1,ni
      do 150 jx=1,nj2
      fx(nj+1-jx,iy)=-fx(jx,iy)
      fy(nj+1-jx,iy)= fy(jx,iy)
      fz(nj+1-jx,iy)= fz(jx,iy)
 150  continue
c
c   compute the magnitude of the force for masking the stress
c
      do 155 iy=1,ni
      do 155 jx=1,nj
      fxt=fx(jx,iy)
      fyt=fy(jx,iy)
      fzt=fz(jx,iy)
      f22=fxt*fxt+fyt*fyt+fzt*fzt
      fm(jx,iy)=0.0
      if(f22.gt.0.05) fm(jx,iy)=1.
 155  continue

c
c  take the fourier transform of the forces
c
      call fourt(fx,n,2,-1,0,work,nwork)
      call fourt(fy,n,2,-1,0,work,nwork)
      call fourt(fz,n,2,-1,0,work,nwork)
c
c  make the coefficients for the visco-elastic half space
c
      call coefan(eta,time,nl,An,namx,rla,rmu)
c
c  construct the fault solution
c
      do 255 i=1,ni
      ky=-(i-1)/height
      if(i.ge.ni2) ky= (ni-i+1)/height
      do 255 j=1,nj2
      kx=(j-1)/width
      cfkx=fkx(j,i)
      cfky=fky(j,i)
      cfkz=fkz(j,i)
      call fvisco(kx,ky,thk,d1,d2,zobs,An,rla,rmu,namx,
     +                                  cfkx,cfky,cfkz,
     +                                    cuk,cvk,cwk)
c
c  now either output displacement or stress
c
      uk(j,i)=fac*rmu1*cuk/(ni*nj)
      vk(j,i)=fac*rmu1*cvk/(ni*nj)
      wk(j,i)=fac*rmu1*cwk/(ni*nj)

c
c  compute the stress if requested. 
c
      if(istr.ge.1) then
        cux=cmplx(0.,2.*pi*kx)*uk(j,i)
        cvy=cmplx(0.,2.*pi*ky)*vk(j,i)
        cuy=cmplx(0.,2.*pi*ky)*uk(j,i)
        cvx=cmplx(0.,2.*pi*kx)*vk(j,i)
        cwz=-rlam1*(cux+cvy)/(rlam1+2.*rmu1)

c  assume units are km so divide by 1000
c  need another factor of 1000 because slip is in mm/yr
c
        uk(j,i)=1.e-6*((rlam1+2*rmu1)*cux+rlam1*(cvy+cwz))
        vk(j,i)=1.e-6*((rlam1+2*rmu1)*cvy+rlam1*(cux+cwz))
        wk(j,i)=1.e-6*(rmu1*(cuy+cvx))
      endif
 255  continue
c
c  do the inverse fft's
c
      call fourt(u,n,2,1,-1,work,nwork)
      call fourt(v,n,2,1,-1,work,nwork)
      call fourt(w,n,2,1,-1,work,nwork)
c
c  compute the dummy value.  we'll make it very big.
c
      rland=1.e13
      rdum=1.e13
c
c  compute the coulomb stress if requested
c
      if(istr.eq.2) then
       do 260 i=1,ni
        do 260 j=1,nj
        txx=u(j,i)
        tyy=v(j,i)
        txy=w(j,i)
        call coulomb(txx,tyy,txy,fr,psi,rnorm,rshr,rcoul)
        u(j,i)=rnorm
        v(j,i)=rshr
        w(j,i)=rcoul
c
c  mask the stress in the fault zone make it VERY big 1.0e12
c
c        if(fm(j,i).gt.0.) then
c          u(j,i)=1.0e12
c          v(j,i)=1.0e12
c          w(j,i)=1.0e12
c        endif
 260  continue
      endif

c
c  write 3 grd files 
c
      rln0=0.
      rlt0=0.
      ddx=dx
      ddy=dx
      call writegrd(u,nj,ni,rln0,rlt0,ddx,ddy, 
     +              rland,rdum,fxdisp,fxdisp)
      call writegrd(v,nj,ni,rln0,rlt0,ddx,ddy,
     +              rland,rdum,fydisp,fydisp)
      call writegrd(w,nj,ni,rln0,rlt0,ddx,ddy,
     +              rland,rdum,fzdisp,fzdisp)

      stop
      end
